@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2021 TileDB Inc.
+ * @copyright Copyright (c) 2021-2024 TileDB Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -62,12 +62,6 @@ struct Attributesfx {
   // Vector of supported filesystems
   const std::vector<std::unique_ptr<SupportedFs>> fs_vec_;
 
-  // Serialization parameters
-  bool serialize_ = false;
-  bool refactored_query_v2_ = false;
-  // Buffers to allocate on server side for serialized queries
-  tiledb::test::ServerQueryBuffers server_buffers_;
-
   // Functions
   Attributesfx();
   ~Attributesfx();
@@ -77,7 +71,6 @@ struct Attributesfx {
       const std::string& path,
       const std::string& attr_name,
       tiledb_datatype_t attr_type);
-  static std::string random_name(const std::string& prefix);
 };
 
 Attributesfx::Attributesfx()
@@ -105,13 +98,6 @@ void Attributesfx::remove_temp_dir(const std::string& path) {
     REQUIRE(tiledb_vfs_remove_dir(ctx_, vfs_, path.c_str()) == TILEDB_OK);
 }
 
-std::string Attributesfx::random_name(const std::string& prefix) {
-  std::stringstream ss;
-  ss << prefix << "-" << std::this_thread::get_id() << "-"
-     << TILEDB_TIMESTAMP_NOW_MS;
-  return ss.str();
-}
-
 void Attributesfx::create_dense_vector(
     const std::string& path,
     const std::string& attr_name,
@@ -164,7 +150,7 @@ void Attributesfx::create_dense_vector(
 TEST_CASE_METHOD(
     Attributesfx,
     "C API: Test attributes with illegal filesystem characters in the name",
-    "[capi][attributes][illegal_name]") {
+    "[capi][attributes][illegal_name][rest]") {
   const std::vector<std::string> attr_names = {
       "miles!hour",  "miles#hour", "miles$hour", "miles%hour",  "miles&hour",
       "miles'hour",  "miles(hour", "miles)hour", "miles*hour",  "miles+hour",
@@ -172,25 +158,12 @@ TEST_CASE_METHOD(
       "miles?hour",  "miles@hour", "miles[hour", "miles]hour",  "miles[hour",
       "miles\"hour", "miles<hour", "miles>hour", "miles\\hour", "miles|hour"};
 
-  SECTION("no serialization") {
-    serialize_ = false;
-  }
-#ifdef TILEDB_SERIALIZATION
-  SECTION("serialization enabled") {
-    serialize_ = true;
-    refactored_query_v2_ = GENERATE(true, false);
-  }
-#endif
-
+  size_t num = 0;
   for (const auto& attr_name : attr_names) {
     for (const auto& fs : fs_vec_) {
       std::string temp_dir = fs->temp_dir();
-      std::string array_name = temp_dir + "array-illegal-char";
-      // serialization is not supported for memfs arrays
-      if (serialize_ &&
-          tiledb::sm::utils::parse::starts_with(array_name, "mem://")) {
-        continue;
-      }
+      std::string array_name = vfs_array_uri(
+          fs, temp_dir + "array-illegal-char" + std::to_string(num++));
 
       // Create new TileDB context with file lock config disabled, rest the
       // same.
@@ -233,13 +206,7 @@ TEST_CASE_METHOD(
       rc = tiledb_query_set_data_buffer(
           ctx_, query, attr_name.c_str(), buffer_a1, &buffer_a1_size);
       CHECK(rc == TILEDB_OK);
-      rc = tiledb::test::submit_query_wrapper(
-          ctx_,
-          array_name,
-          &query,
-          server_buffers_,
-          serialize_,
-          refactored_query_v2_);
+      rc = tiledb_query_submit_and_finalize(ctx_, query);
       CHECK(rc == TILEDB_OK);
 
       // Close array and clean up
@@ -267,13 +234,8 @@ TEST_CASE_METHOD(
       rc = tiledb_query_set_data_buffer(
           ctx_, query, attr_name.c_str(), buffer_read, &buffer_read_size);
       CHECK(rc == TILEDB_OK);
-      rc = submit_query_wrapper(
-          ctx_,
-          array_name,
-          &query,
-          server_buffers_,
-          serialize_,
-          refactored_query_v2_);
+      rc = tiledb_query_submit(ctx_, query);
+
       CHECK(rc == TILEDB_OK);
 
       // Close array and clean up
@@ -288,51 +250,35 @@ TEST_CASE_METHOD(
       CHECK(buffer_read_size == sizeof(buffer_read_c));
 
       remove_temp_dir(temp_dir);
+      CHECK(vfs_test_close(fs_vec_, ctx_, vfs_).ok());
     }
   }
 }
 
 TEST_CASE_METHOD(
     Attributesfx,
-    "C API: Test attributes with tiledb_blob datatype",
-    "[capi][attributes][tiledb_blob]") {
-  SECTION("no serialization") {
-    serialize_ = false;
-  }
-#ifdef TILEDB_SERIALIZATION
-  SECTION("serialization enabled") {
-    serialize_ = true;
-    refactored_query_v2_ = GENERATE(true, false);
-  }
-#endif
+    "C API: Test attributes with std::byte",
+    "[capi][attributes][byte][rest-fails][sc-40489]") {
+  auto datatype = GENERATE(TILEDB_BLOB, TILEDB_GEOM_WKB, TILEDB_GEOM_WKT);
+
   for (const auto& fs : fs_vec_) {
     std::string temp_dir = fs->temp_dir();
-    std::string array_name = temp_dir;
-    // serialization is not supported for memfs arrays
-    if (serialize_ &&
-        tiledb::sm::utils::parse::starts_with(array_name, "mem://")) {
-      continue;
-    }
-
-    std::string attr_name = "attr";
+    std::string array_name = vfs_array_uri(fs, temp_dir + "byte-attribute");
+    std::string attr_name = "a";
 
     // Create new TileDB context with file lock config disabled, rest the
     // same.
     tiledb_ctx_free(&ctx_);
     tiledb_vfs_free(&vfs_);
-
     tiledb_config_t* config = nullptr;
     tiledb_error_t* error = nullptr;
     REQUIRE(tiledb_config_alloc(&config, &error) == TILEDB_OK);
     REQUIRE(error == nullptr);
-
     REQUIRE(vfs_test_init(fs_vec_, &ctx_, &vfs_, config).ok());
-
     tiledb_config_free(&config);
 
     create_temp_dir(temp_dir);
-
-    create_dense_vector(array_name, attr_name, TILEDB_BLOB);
+    create_dense_vector(array_name, attr_name, datatype);
 
     // Prepare cell buffers
     uint8_t buffer_write[] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10};
@@ -357,13 +303,7 @@ TEST_CASE_METHOD(
     rc = tiledb_query_set_data_buffer(
         ctx_, query, attr_name.c_str(), buffer_write, &buffer_write_size);
     CHECK(rc == TILEDB_OK);
-    rc = submit_query_wrapper(
-        ctx_,
-        array_name,
-        &query,
-        server_buffers_,
-        serialize_,
-        refactored_query_v2_);
+    rc = tiledb_query_submit_and_finalize(ctx_, query);
     CHECK(rc == TILEDB_OK);
 
     // Close array and clean up
@@ -372,12 +312,92 @@ TEST_CASE_METHOD(
     tiledb_array_free(&array);
     tiledb_query_free(&query);
 
+    uint64_t ts_open = 0;
+    if (datatype == TILEDB_BLOB) {
+      // For the BLOB datatype, test with and without schema evolution.
+      auto evolve = GENERATE(true, false);
+      if (evolve) {
+        // Ensure the BLOB type can evolve to both GEOM types.
+        auto new_type = GENERATE(TILEDB_GEOM_WKB, TILEDB_GEOM_WKT);
+        // Add a second attribute on the schema and drop the original.
+        tiledb_array_schema_evolution_t* schema_evolution;
+        rc = tiledb_array_schema_evolution_alloc(ctx_, &schema_evolution);
+        REQUIRE(rc == TILEDB_OK);
+        tiledb_attribute_t* b;
+        rc = tiledb_attribute_alloc(ctx_, "b", TILEDB_BLOB, &b);
+        REQUIRE(rc == TILEDB_OK);
+        rc = tiledb_array_schema_evolution_add_attribute(
+            ctx_, schema_evolution, b);
+        REQUIRE(rc == TILEDB_OK);
+        rc = tiledb_array_schema_evolution_drop_attribute(
+            ctx_, schema_evolution, attr_name.c_str());
+        REQUIRE(rc == TILEDB_OK);
+        // Set timestamp to avoid race condition
+        ts_open = tiledb_timestamp_now_ms();
+        ts_open = ts_open + 1;
+        rc = tiledb_array_schema_evolution_set_timestamp_range(
+            ctx_, schema_evolution, ts_open, ts_open);
+        rc = tiledb_array_evolve(ctx_, array_name.c_str(), schema_evolution);
+        REQUIRE(rc == TILEDB_OK);
+
+        // Add back the original attribute as new_type and drop "b".
+        tiledb_array_schema_evolution_t* schema_evolution2;
+        rc = tiledb_array_schema_evolution_alloc(ctx_, &schema_evolution2);
+        REQUIRE(rc == TILEDB_OK);
+        tiledb_attribute_t* attr;
+        rc = tiledb_attribute_alloc(ctx_, attr_name.c_str(), new_type, &attr);
+        REQUIRE(rc == TILEDB_OK);
+        rc = tiledb_array_schema_evolution_add_attribute(
+            ctx_, schema_evolution2, attr);
+        REQUIRE(rc == TILEDB_OK);
+        rc = tiledb_array_schema_evolution_drop_attribute(
+            ctx_, schema_evolution2, "b");
+        REQUIRE(rc == TILEDB_OK);
+        // Set timestamp to avoid race condition
+        ts_open = tiledb_timestamp_now_ms();
+        ts_open = ts_open + 2;
+        rc = tiledb_array_schema_evolution_set_timestamp_range(
+            ctx_, schema_evolution2, ts_open, ts_open);
+#ifdef TILEDB_SERIALIZATION
+        // Serialize the array schema evolution
+        tiledb_buffer_t* buffer;
+        rc = tiledb_serialize_array_schema_evolution(
+            ctx_,
+            schema_evolution2,
+            (tiledb_serialization_type_t)tiledb::sm::SerializationType::CAPNP,
+            0,
+            &buffer);
+        REQUIRE(rc == TILEDB_OK);
+        rc = tiledb_deserialize_array_schema_evolution(
+            ctx_,
+            buffer,
+            (tiledb_serialization_type_t)tiledb::sm::SerializationType::CAPNP,
+            1,
+            &schema_evolution2);
+        REQUIRE(rc == TILEDB_OK);
+        tiledb_buffer_free(&buffer);
+#endif
+        rc = tiledb_array_evolve(ctx_, array_name.c_str(), schema_evolution2);
+        REQUIRE(rc == TILEDB_OK);
+
+        // Clean up
+        tiledb_attribute_free(&b);
+        tiledb_attribute_free(&attr);
+        tiledb_array_schema_evolution_free(&schema_evolution);
+        tiledb_array_schema_evolution_free(&schema_evolution2);
+      }
+    }
+
     int buffer_read[10];
     uint64_t buffer_read_size = sizeof(buffer_read);
 
     // Open array
     rc = tiledb_array_alloc(ctx_, array_name.c_str(), &array);
     CHECK(rc == TILEDB_OK);
+    if (ts_open != 0) {
+      rc = tiledb_array_set_open_timestamp_end(ctx_, array, ts_open + 2);
+      REQUIRE(rc == TILEDB_OK);
+    }
     rc = tiledb_array_open(ctx_, array, TILEDB_READ);
     CHECK(rc == TILEDB_OK);
 
@@ -391,13 +411,7 @@ TEST_CASE_METHOD(
     rc = tiledb_query_set_data_buffer(
         ctx_, query, attr_name.c_str(), buffer_read, &buffer_read_size);
     CHECK(rc == TILEDB_OK);
-    rc = submit_query_wrapper(
-        ctx_,
-        array_name,
-        &query,
-        server_buffers_,
-        serialize_,
-        refactored_query_v2_);
+    rc = tiledb_query_submit(ctx_, query);
     CHECK(rc == TILEDB_OK);
 
     // Close array and clean up
@@ -422,24 +436,10 @@ TEST_CASE_METHOD(
 TEST_CASE_METHOD(
     Attributesfx,
     "C API: Test attributes with tiledb_bool datatype",
-    "[capi][attributes][tiledb_bool]") {
-  SECTION("no serialization") {
-    serialize_ = false;
-  }
-#ifdef TILEDB_SERIALIZATION
-  SECTION("serialization enabled") {
-    serialize_ = true;
-    refactored_query_v2_ = GENERATE(true, false);
-  }
-#endif
+    "[capi][attributes][tiledb_bool][rest]") {
   for (const auto& fs : fs_vec_) {
     std::string temp_dir = fs->temp_dir();
-    std::string array_name = temp_dir;
-    // serialization is not supported for memfs arrays
-    if (serialize_ &&
-        tiledb::sm::utils::parse::starts_with(array_name, "mem://")) {
-      continue;
-    }
+    std::string array_name = vfs_array_uri(fs, temp_dir + "bool-attribute");
 
     std::string attr_name = "attr";
 
@@ -484,13 +484,7 @@ TEST_CASE_METHOD(
     rc = tiledb_query_set_data_buffer(
         ctx_, query, attr_name.c_str(), buffer_write, &buffer_write_size);
     CHECK(rc == TILEDB_OK);
-    rc = submit_query_wrapper(
-        ctx_,
-        array_name,
-        &query,
-        server_buffers_,
-        serialize_,
-        refactored_query_v2_);
+    rc = tiledb_query_submit_and_finalize(ctx_, query);
     CHECK(rc == TILEDB_OK);
 
     // Close array and clean up
@@ -518,13 +512,7 @@ TEST_CASE_METHOD(
     rc = tiledb_query_set_data_buffer(
         ctx_, query, attr_name.c_str(), buffer_read, &buffer_read_size);
     CHECK(rc == TILEDB_OK);
-    rc = submit_query_wrapper(
-        ctx_,
-        array_name,
-        &query,
-        server_buffers_,
-        serialize_,
-        refactored_query_v2_);
+    rc = tiledb_query_submit(ctx_, query);
     CHECK(rc == TILEDB_OK);
 
     // Close array and clean up
