@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2021-2022 TileDB, Inc.
+ * @copyright Copyright (c) 2021-2024 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -32,13 +32,14 @@
 
 #include "tiledb/sm/query/query_condition.h"
 #include "tiledb/common/logger.h"
+#include "tiledb/common/memory_tracker.h"
 #include "tiledb/sm/enums/datatype.h"
 #include "tiledb/sm/enums/query_condition_combination_op.h"
 #include "tiledb/sm/enums/query_condition_op.h"
+#include "tiledb/sm/fragment/fragment_identifier.h"
 #include "tiledb/sm/fragment/fragment_metadata.h"
 #include "tiledb/sm/misc/utils.h"
 #include "tiledb/sm/query/readers/result_cell_slab.h"
-#include "tiledb/storage_format/uri/parse_uri.h"
 
 #include <algorithm>
 #include <functional>
@@ -51,8 +52,7 @@
 
 using namespace tiledb::common;
 
-namespace tiledb {
-namespace sm {
+namespace tiledb::sm {
 
 QueryCondition::QueryCondition() {
 }
@@ -127,6 +127,12 @@ Status QueryCondition::init(
     return Status_QueryConditionError("Cannot reinitialize query condition");
   }
 
+  if (op == QueryConditionOp::ALWAYS_TRUE ||
+      op == QueryConditionOp::ALWAYS_FALSE) {
+    auto op_str = query_condition_op_str(op);
+    throw std::runtime_error("Invalid use of internal operation: " + op_str);
+  }
+
   // AST Construction.
   tree_ = tdb_unique_ptr<ASTNode>(tdb_new(
       ASTNodeVal, field_name, condition_value, condition_value_size, op));
@@ -210,12 +216,8 @@ uint64_t QueryCondition::condition_timestamp() const {
     return 0;
   }
 
-  std::pair<uint64_t, uint64_t> timestamps;
-  if (!utils::parse::get_timestamp_range(URI(condition_marker_), &timestamps)
-           .ok()) {
-    throw std::logic_error("Error parsing condition marker.");
-  }
-
+  FragmentID fragment_id{condition_marker_};
+  auto timestamps{fragment_id.timestamp_range()};
   return timestamps.first;
 }
 
@@ -487,6 +489,22 @@ struct QueryCondition::BinaryCmpNullChecks<uint8_t*, QueryConditionOp::NE> {
   }
 };
 
+/** Partial template specialization for `QueryConditionOp::LT`. */
+template <typename T>
+struct QueryCondition::BinaryCmpNullChecks<T, QueryConditionOp::ALWAYS_TRUE> {
+  static inline bool cmp(const void* lhs, uint64_t, const void*, uint64_t) {
+    return lhs != nullptr;
+  }
+};
+
+/** Partial template specialization for `QueryConditionOp::LT`. */
+template <typename T>
+struct QueryCondition::BinaryCmpNullChecks<T, QueryConditionOp::ALWAYS_FALSE> {
+  static inline bool cmp(const void*, uint64_t, const void*, uint64_t) {
+    return false;
+  }
+};
+
 /** Partial template specialization for `QueryConditionOp::LT`. */
 template <typename T>
 struct QueryCondition::BinaryCmpNullChecks<T, QueryConditionOp::LT> {
@@ -599,7 +617,7 @@ void QueryCondition::apply_ast_node(
     const ByteVecValue& fill_value,
     const std::vector<ResultCellSlab>& result_cell_slabs,
     CombinationOp combination_op,
-    std::vector<uint8_t>& result_cell_bitmap) const {
+    tdb::pmr::vector<uint8_t>& result_cell_bitmap) const {
   const std::string& field_name = node->get_field_name();
   const void* condition_value_content = node->get_value_ptr();
   const size_t condition_value_size = node->get_value_size();
@@ -751,8 +769,32 @@ void QueryCondition::apply_ast_node(
     const ByteVecValue& fill_value,
     const std::vector<ResultCellSlab>& result_cell_slabs,
     CombinationOp combination_op,
-    std::vector<uint8_t>& result_cell_bitmap) const {
+    tdb::pmr::vector<uint8_t>& result_cell_bitmap) const {
   switch (node->get_op()) {
+    case QueryConditionOp::ALWAYS_TRUE:
+      apply_ast_node<T, QueryConditionOp::ALWAYS_TRUE, CombinationOp>(
+          node,
+          fragment_metadata,
+          stride,
+          var_size,
+          nullable,
+          fill_value,
+          result_cell_slabs,
+          combination_op,
+          result_cell_bitmap);
+      break;
+    case QueryConditionOp::ALWAYS_FALSE:
+      apply_ast_node<T, QueryConditionOp::ALWAYS_FALSE, CombinationOp>(
+          node,
+          fragment_metadata,
+          stride,
+          var_size,
+          nullable,
+          fill_value,
+          result_cell_slabs,
+          combination_op,
+          result_cell_bitmap);
+      break;
     case QueryConditionOp::LT:
       apply_ast_node<T, QueryConditionOp::LT, CombinationOp>(
           node,
@@ -866,7 +908,7 @@ void QueryCondition::apply_ast_node(
     const uint64_t stride,
     const std::vector<ResultCellSlab>& result_cell_slabs,
     CombinationOp combination_op,
-    std::vector<uint8_t>& result_cell_bitmap) const {
+    tdb::pmr::vector<uint8_t>& result_cell_bitmap) const {
   std::string node_field_name = node->get_field_name();
 
   const auto nullable = array_schema.is_nullable(node_field_name);
@@ -1091,6 +1133,8 @@ void QueryCondition::apply_ast_node(
     } break;
     case Datatype::ANY:
     case Datatype::BLOB:
+    case Datatype::GEOM_WKB:
+    case Datatype::GEOM_WKT:
     case Datatype::STRING_UTF16:
     case Datatype::STRING_UTF32:
     case Datatype::STRING_UCS2:
@@ -1098,9 +1142,8 @@ void QueryCondition::apply_ast_node(
     default:
       throw std::runtime_error(
           "QueryCondition::apply_ast_node: Cannot perform query comparison; "
-          "Unsupported query "
-          "conditional type on " +
-          node->get_field_name());
+          "Unsupported datatype " +
+          datatype_str(type) + " on " + node->get_field_name());
   }
 
   return;
@@ -1109,12 +1152,13 @@ void QueryCondition::apply_ast_node(
 template <typename CombinationOp>
 void QueryCondition::apply_tree(
     const tdb_unique_ptr<ASTNode>& node,
-    const ArraySchema& array_schema,
+    const QueryCondition::Params& params,
     const std::vector<shared_ptr<FragmentMetadata>>& fragment_metadata,
     uint64_t stride,
     const std::vector<ResultCellSlab>& result_cell_slabs,
     CombinationOp combination_op,
-    std::vector<uint8_t>& result_cell_bitmap) const {
+    tdb::pmr::vector<uint8_t>& result_cell_bitmap) const {
+  const auto& array_schema = params.GetSchema();
   if (!node->is_expr()) {
     apply_ast_node(
         node,
@@ -1149,7 +1193,7 @@ void QueryCondition::apply_tree(
           for (const auto& child : node->get_children()) {
             apply_tree(
                 child,
-                array_schema,
+                params,
                 fragment_metadata,
                 stride,
                 result_cell_slabs,
@@ -1161,12 +1205,15 @@ void QueryCondition::apply_tree(
         } else if constexpr (std::is_same_v<
                                  CombinationOp,
                                  std::logical_or<uint8_t>>) {
-          std::vector<uint8_t> combination_op_bitmap(result_bitmap_size, 1);
+          auto resource = params.GetMemoryTracker()->get_resource(
+              MemoryType::QUERY_CONDITION);
+          tdb::pmr::vector<uint8_t> combination_op_bitmap(
+              result_bitmap_size, 1, resource);
 
           for (const auto& child : node->get_children()) {
             apply_tree(
                 child,
-                array_schema,
+                params,
                 fragment_metadata,
                 stride,
                 result_cell_slabs,
@@ -1185,12 +1232,15 @@ void QueryCondition::apply_tree(
          *                        = a /\ (cl1'(q; cl2'(q; 0)))
          */
       case QueryConditionCombinationOp::OR: {
-        std::vector<uint8_t> combination_op_bitmap(result_bitmap_size, 0);
+        auto resource = params.GetMemoryTracker()->get_resource(
+            MemoryType::QUERY_CONDITION);
+        tdb::pmr::vector<uint8_t> combination_op_bitmap(
+            result_bitmap_size, 0, resource);
 
         for (const auto& child : node->get_children()) {
           apply_tree(
               child,
-              array_schema,
+              params,
               fragment_metadata,
               stride,
               result_cell_slabs,
@@ -1215,7 +1265,7 @@ void QueryCondition::apply_tree(
 }
 
 Status QueryCondition::apply(
-    const ArraySchema& array_schema,
+    const QueryCondition::Params& params,
     const std::vector<shared_ptr<FragmentMetadata>>& fragment_metadata,
     std::vector<ResultCellSlab>& result_cell_slabs,
     const uint64_t stride) const {
@@ -1228,10 +1278,13 @@ Status QueryCondition::apply(
     total_lengths += elem.length_;
   }
 
-  std::vector<uint8_t> result_cell_bitmap(total_lengths, 1);
+  auto resource =
+      params.GetMemoryTracker()->get_resource(MemoryType::QUERY_CONDITION);
+  tdb::pmr::vector<uint8_t> result_cell_bitmap(total_lengths, 1, resource);
+
   apply_tree(
       tree_,
-      array_schema,
+      params,
       fragment_metadata,
       stride,
       result_cell_slabs,
@@ -1444,6 +1497,34 @@ void QueryCondition::apply_ast_node_dense(
     const void* cell_slab_coords,
     span<uint8_t> result_buffer) const {
   switch (node->get_op()) {
+    case QueryConditionOp::ALWAYS_TRUE:
+      apply_ast_node_dense<T, QueryConditionOp::ALWAYS_TRUE, CombinationOp>(
+          node,
+          array_schema,
+          result_tile,
+          start,
+          src_cell,
+          stride,
+          var_size,
+          nullable,
+          combination_op,
+          cell_slab_coords,
+          result_buffer);
+      break;
+    case QueryConditionOp::ALWAYS_FALSE:
+      apply_ast_node_dense<T, QueryConditionOp::ALWAYS_FALSE, CombinationOp>(
+          node,
+          array_schema,
+          result_tile,
+          start,
+          src_cell,
+          stride,
+          var_size,
+          nullable,
+          combination_op,
+          cell_slab_coords,
+          result_buffer);
+      break;
     case QueryConditionOp::LT:
       apply_ast_node_dense<T, QueryConditionOp::LT, CombinationOp>(
           node,
@@ -1847,6 +1928,8 @@ void QueryCondition::apply_ast_node_dense(
     } break;
     case Datatype::ANY:
     case Datatype::BLOB:
+    case Datatype::GEOM_WKB:
+    case Datatype::GEOM_WKT:
     case Datatype::STRING_UTF16:
     case Datatype::STRING_UTF32:
     case Datatype::STRING_UCS2:
@@ -1854,16 +1937,15 @@ void QueryCondition::apply_ast_node_dense(
     default:
       throw std::runtime_error(
           "Cannot perform query comparison; Unsupported query conditional "
-          "type "
-          "on " +
-          node->get_field_name());
+          "type " +
+          datatype_str(type) + " on " + node->get_field_name());
   }
 }
 
 template <typename CombinationOp>
 void QueryCondition::apply_tree_dense(
     const tdb_unique_ptr<ASTNode>& node,
-    const ArraySchema& array_schema,
+    const QueryCondition::Params& params,
     ResultTile* result_tile,
     const uint64_t start,
     const uint64_t src_cell,
@@ -1874,7 +1956,7 @@ void QueryCondition::apply_tree_dense(
   if (!node->is_expr()) {
     apply_ast_node_dense(
         node,
-        array_schema,
+        params.GetSchema(),
         result_tile,
         start,
         src_cell,
@@ -1907,7 +1989,7 @@ void QueryCondition::apply_tree_dense(
           for (const auto& child : node->get_children()) {
             apply_tree_dense(
                 child,
-                array_schema,
+                params,
                 result_tile,
                 start,
                 src_cell,
@@ -1921,14 +2003,17 @@ void QueryCondition::apply_tree_dense(
         } else if constexpr (std::is_same_v<
                                  CombinationOp,
                                  std::logical_or<uint8_t>>) {
-          std::vector<uint8_t> combination_op_bitmap(result_buffer_size, 1);
+          auto resource = params.GetMemoryTracker()->get_resource(
+              MemoryType::QUERY_CONDITION);
+          tdb::pmr::vector<uint8_t> combination_op_bitmap(
+              result_buffer_size, 1, resource);
           span<uint8_t> combination_op_span(
               combination_op_bitmap.data(), result_buffer_size);
 
           for (const auto& child : node->get_children()) {
             apply_tree_dense(
                 child,
-                array_schema,
+                params,
                 result_tile,
                 start,
                 src_cell,
@@ -1948,13 +2033,16 @@ void QueryCondition::apply_tree_dense(
          *                        = a /\ (cl1'(q; cl2'(q; 0)))
          */
       case QueryConditionCombinationOp::OR: {
-        std::vector<uint8_t> combination_op_bitmap(result_buffer_size, 0);
+        auto resource = params.GetMemoryTracker()->get_resource(
+            MemoryType::QUERY_CONDITION);
+        tdb::pmr::vector<uint8_t> combination_op_bitmap(
+            result_buffer_size, 0, resource);
         span<uint8_t> combination_op_span(
             combination_op_bitmap.data(), result_buffer_size);
         for (const auto& child : node->get_children()) {
           apply_tree_dense(
               child,
-              array_schema,
+              params,
               result_tile,
               start,
               src_cell,
@@ -1981,7 +2069,7 @@ void QueryCondition::apply_tree_dense(
 }
 
 Status QueryCondition::apply_dense(
-    const ArraySchema& array_schema,
+    const QueryCondition::Params& params,
     ResultTile* result_tile,
     const uint64_t start,
     const uint64_t length,
@@ -1997,7 +2085,7 @@ Status QueryCondition::apply_dense(
   span<uint8_t> result_span(result_buffer + start, length);
   apply_tree_dense(
       tree_,
-      array_schema,
+      params,
       result_tile,
       start,
       src_cell,
@@ -2208,6 +2296,22 @@ struct QueryCondition::BinaryCmp<uint8_t*, QueryConditionOp::NE> {
   }
 };
 
+/** Partial template specialization for `QueryConditionOp::LT`. */
+template <typename T>
+struct QueryCondition::BinaryCmp<T, QueryConditionOp::ALWAYS_TRUE> {
+  static inline bool cmp(const void*, uint64_t, const void*, uint64_t) {
+    return true;
+  }
+};
+
+/** Partial template specialization for `QueryConditionOp::LT`. */
+template <typename T>
+struct QueryCondition::BinaryCmp<T, QueryConditionOp::ALWAYS_FALSE> {
+  static inline bool cmp(const void*, uint64_t, const void*, uint64_t) {
+    return false;
+  }
+};
+
 /** Partial template specialization for `QueryConditionOp::LT`. */
 template <typename T>
 struct QueryCondition::BinaryCmp<T, QueryConditionOp::LT> {
@@ -2298,7 +2402,7 @@ void QueryCondition::apply_ast_node_sparse(
     ResultTile& result_tile,
     const bool var_size,
     CombinationOp combination_op,
-    std::vector<BitmapType>& result_bitmap) const {
+    tdb::pmr::vector<BitmapType>& result_bitmap) const {
   const auto tile_tuple = result_tile.tile_tuple(node->get_field_name());
   const void* condition_value_content = node->get_value_ptr();
   const size_t condition_value_size = node->get_value_size();
@@ -2386,8 +2490,24 @@ void QueryCondition::apply_ast_node_sparse(
     ResultTile& result_tile,
     const bool var_size,
     CombinationOp combination_op,
-    std::vector<BitmapType>& result_bitmap) const {
+    tdb::pmr::vector<BitmapType>& result_bitmap) const {
   switch (node->get_op()) {
+    case QueryConditionOp::ALWAYS_TRUE:
+      apply_ast_node_sparse<
+          T,
+          QueryConditionOp::ALWAYS_TRUE,
+          BitmapType,
+          CombinationOp,
+          nullable>(node, result_tile, var_size, combination_op, result_bitmap);
+      break;
+    case QueryConditionOp::ALWAYS_FALSE:
+      apply_ast_node_sparse<
+          T,
+          QueryConditionOp::ALWAYS_FALSE,
+          BitmapType,
+          CombinationOp,
+          nullable>(node, result_tile, var_size, combination_op, result_bitmap);
+      break;
     case QueryConditionOp::LT:
       apply_ast_node_sparse<
           T,
@@ -2466,7 +2586,7 @@ void QueryCondition::apply_ast_node_sparse(
     const bool var_size,
     const bool nullable,
     CombinationOp combination_op,
-    std::vector<BitmapType>& result_bitmap) const {
+    tdb::pmr::vector<BitmapType>& result_bitmap) const {
   if (nullable) {
     apply_ast_node_sparse<T, BitmapType, CombinationOp, std::true_type>(
         node, result_tile, var_size, combination_op, result_bitmap);
@@ -2482,7 +2602,7 @@ void QueryCondition::apply_ast_node_sparse(
     const ArraySchema& array_schema,
     ResultTile& result_tile,
     CombinationOp combination_op,
-    std::vector<BitmapType>& result_bitmap) const {
+    tdb::pmr::vector<BitmapType>& result_bitmap) const {
   std::string node_field_name = node->get_field_name();
   if (!array_schema.is_field(node_field_name)) {
     std::fill(result_bitmap.begin(), result_bitmap.end(), 0);
@@ -2621,6 +2741,8 @@ void QueryCondition::apply_ast_node_sparse(
     } break;
     case Datatype::ANY:
     case Datatype::BLOB:
+    case Datatype::GEOM_WKB:
+    case Datatype::GEOM_WKT:
     case Datatype::STRING_UTF16:
     case Datatype::STRING_UTF32:
     case Datatype::STRING_UCS2:
@@ -2636,10 +2758,11 @@ void QueryCondition::apply_ast_node_sparse(
 template <typename BitmapType, typename CombinationOp>
 void QueryCondition::apply_tree_sparse(
     const tdb_unique_ptr<ASTNode>& node,
-    const ArraySchema& array_schema,
+    const QueryCondition::Params& params,
     ResultTile& result_tile,
     CombinationOp combination_op,
-    std::vector<BitmapType>& result_bitmap) const {
+    tdb::pmr::vector<BitmapType>& result_bitmap) const {
+  const auto& array_schema = params.GetSchema();
   if (!node->is_expr()) {
     apply_ast_node_sparse<BitmapType>(
         node, array_schema, result_tile, combination_op, result_bitmap);
@@ -2669,7 +2792,7 @@ void QueryCondition::apply_tree_sparse(
           for (const auto& child : node->get_children()) {
             apply_tree_sparse<BitmapType>(
                 child,
-                array_schema,
+                params,
                 result_tile,
                 std::multiplies<BitmapType>(),
                 result_bitmap);
@@ -2678,12 +2801,15 @@ void QueryCondition::apply_tree_sparse(
           // Handle the cl'(q, a) case.
           // This cases on whether the combination op = OR.
         } else if constexpr (std::is_same_v<CombinationOp, QCMax<BitmapType>>) {
-          std::vector<BitmapType> combination_op_bitmap(result_bitmap_size, 1);
+          auto resource = params.GetMemoryTracker()->get_resource(
+              MemoryType::QUERY_CONDITION);
+          tdb::pmr::vector<BitmapType> combination_op_bitmap(
+              result_bitmap_size, 1, resource);
 
           for (const auto& child : node->get_children()) {
             apply_tree_sparse<BitmapType>(
                 child,
-                array_schema,
+                params,
                 result_tile,
                 std::multiplies<BitmapType>(),
                 combination_op_bitmap);
@@ -2700,12 +2826,15 @@ void QueryCondition::apply_tree_sparse(
          *                        = a /\ (cl1'(q; cl2'(q; 0)))
          */
       case QueryConditionCombinationOp::OR: {
-        std::vector<BitmapType> combination_op_bitmap(result_bitmap_size, 0);
+        auto resource = params.GetMemoryTracker()->get_resource(
+            MemoryType::QUERY_CONDITION);
+        tdb::pmr::vector<BitmapType> combination_op_bitmap(
+            result_bitmap_size, 0, resource);
 
         for (const auto& child : node->get_children()) {
           apply_tree_sparse<BitmapType>(
               child,
-              array_schema,
+              params,
               result_tile,
               QCMax<BitmapType>(),
               combination_op_bitmap);
@@ -2729,15 +2858,11 @@ void QueryCondition::apply_tree_sparse(
 
 template <typename BitmapType>
 Status QueryCondition::apply_sparse(
-    const ArraySchema& array_schema,
+    const QueryCondition::Params& params,
     ResultTile& result_tile,
-    std::vector<BitmapType>& result_bitmap) {
+    tdb::pmr::vector<BitmapType>& result_bitmap) {
   apply_tree_sparse<BitmapType>(
-      tree_,
-      array_schema,
-      result_tile,
-      std::multiplies<BitmapType>(),
-      result_bitmap);
+      tree_, params, result_tile, std::multiplies<BitmapType>(), result_bitmap);
 
   return Status::Ok();
 }
@@ -2758,14 +2883,9 @@ uint64_t QueryCondition::condition_index() const {
   return condition_index_;
 }
 
-void QueryCondition::set_ast(tdb_unique_ptr<ASTNode>&& ast) {
-  tree_ = std::move(ast);
-}
-
 // Explicit template instantiations.
 template Status QueryCondition::apply_sparse<uint8_t>(
-    const ArraySchema& array_schema, ResultTile&, std::vector<uint8_t>&);
+    const QueryCondition::Params&, ResultTile&, tdb::pmr::vector<uint8_t>&);
 template Status QueryCondition::apply_sparse<uint64_t>(
-    const ArraySchema& array_schema, ResultTile&, std::vector<uint64_t>&);
-}  // namespace sm
-}  // namespace tiledb
+    const QueryCondition::Params&, ResultTile&, tdb::pmr::vector<uint64_t>&);
+}  // namespace tiledb::sm
