@@ -34,7 +34,6 @@
 #define TILEDB_STRATEGY_BASE_H
 
 #include "tiledb/common/common.h"
-#include "tiledb/common/logger_public.h"
 #include "tiledb/common/status.h"
 #include "tiledb/sm/array_schema/dimension.h"
 #include "tiledb/sm/misc/types.h"
@@ -66,6 +65,8 @@ class StrategyParams {
   /* ********************************* */
 
   StrategyParams(
+      shared_ptr<MemoryTracker> array_memory_tracker,
+      shared_ptr<MemoryTracker> query_memory_tracker,
       StorageManager* storage_manager,
       shared_ptr<OpenedArray> array,
       Config& config,
@@ -75,9 +76,10 @@ class StrategyParams {
       Layout layout,
       std::optional<QueryCondition>& condition,
       DefaultChannelAggregates& default_channel_aggregates,
-      bool skip_checks_serialization,
-      MemoryTracker* memory_tracker)
-      : storage_manager_(storage_manager)
+      bool skip_checks_serialization)
+      : array_memory_tracker_(array_memory_tracker)
+      , query_memory_tracker_(query_memory_tracker)
+      , storage_manager_(storage_manager)
       , array_(array)
       , config_(config)
       , buffers_(buffers)
@@ -86,14 +88,22 @@ class StrategyParams {
       , layout_(layout)
       , condition_(condition)
       , default_channel_aggregates_(default_channel_aggregates)
-      , skip_checks_serialization_(skip_checks_serialization)
-      , memory_tracker_(memory_tracker) {
+      , skip_checks_serialization_(skip_checks_serialization) {
   }
 
   /* ********************************* */
   /*                 API               */
   /* ********************************* */
 
+  /** Return the array memory tracker. */
+  inline shared_ptr<MemoryTracker> array_memory_tracker() {
+    return array_memory_tracker_;
+  }
+
+  inline shared_ptr<MemoryTracker> query_memory_tracker() {
+    return query_memory_tracker_;
+  }
+
   /** Return the storage manager. */
   inline StorageManager* storage_manager() {
     return storage_manager_;
@@ -144,15 +154,17 @@ class StrategyParams {
     return skip_checks_serialization_;
   }
 
-  inline MemoryTracker* memory_tracker() {
-    return memory_tracker_;
-  }
-
  private:
   /* ********************************* */
   /*        PRIVATE ATTRIBUTES         */
   /* ********************************* */
 
+  /** Array Memory tracker. */
+  shared_ptr<MemoryTracker> array_memory_tracker_;
+
+  /** Query Memory tracker. */
+  shared_ptr<MemoryTracker> query_memory_tracker_;
+
   /** Storage manager. */
   StorageManager* storage_manager_;
 
@@ -182,9 +194,6 @@ class StrategyParams {
 
   /** Skip checks for serialization. */
   bool skip_checks_serialization_;
-
-  /** Memory tracker. */
-  MemoryTracker* memory_tracker_;
 };
 
 /** Processes read or write queries. */
@@ -206,7 +215,17 @@ class StrategyBase {
   /* ********************************* */
 
   /** Returns `stats_`. */
-  stats::Stats* stats() const;
+  inline stats::Stats* stats() const {
+    return stats_;
+  }
+
+  /**
+   * Populate the owned stats instance with data.
+   * To be removed when the class will get a C41 constructor.
+   *
+   * @param data Data to populate the stats with.
+   */
+  void set_stats(const stats::StatsData& data);
 
   /** Returns the configured offsets format mode. */
   std::string offsets_mode() const;
@@ -231,6 +250,12 @@ class StrategyBase {
   /*        PROTECTED ATTRIBUTES       */
   /* ********************************* */
 
+  /** The array memory tracker. */
+  shared_ptr<MemoryTracker> array_memory_tracker_;
+
+  /** The query memory tracker. */
+  shared_ptr<MemoryTracker> query_memory_tracker_;
+
   /** The class stats. */
   stats::Stats* stats_;
 
